#!/usr/bin/env python
# -*- coding: UTF8 -*-
"""https://oxfordstrat.com/coasdfASD32/uploads/2016/01/turtle-rules.pdf

Adjusting Trading Size

There will be times when the market does not trend for many months. During these
times, it is possible to lose a significant percentage of the equity of the account.
After large winning trades close out, one might want to increase the size of the equity
used to compute position size.

The Turtles did not trade normal accounts with a running balance based on the initial
equity. We were given notional accounts with a starting equity of zero and a specific
account size. For example, many Turtles received a notional account size of $1,000,000
when we first started trading in February, 1983. This account size was then adjusted
each year at the beginning of the year. It was adjusted up or down depending on the
success of the trader as measured subjectively by Rich. The increase/decrease typically
represented something close to the addition of the gains or losses that were made in
the account during the preceding year.

The Turtles were instructed to decrease the size of the notional account by 20% each
time we went down 10% of the original account. So if a Turtle trading a $1,000,000
account was ever was down 10%, or $100,000, we would then begin trading as if we
had a $800,000 account until such time as we reached the yearly starting equity. If we
lost another 10% (10% of $800,000 or $80,000 for a total loss of $180,000) we were to
reduce the account size by another 20% for a notional account size of $640,000.

There are other, perhaps better strategies for reducing or increasing equity as the
account goes up or down. These are simply the rules that the Turtles used
"""


"""
The Turtles were instructed to decrease the size of the notional account by 20% each
time we went down 10% of the original account. So if a Turtle trading a $1,000,000
account was ever was down 10%, or $100,000, we would then begin trading as if we
had a $800,000 account until such time as we reached the yearly starting equity
"""
def decrease_size(account_size: float, ) -> float:
  """Return the new account size. This function is called if the account has suffered a 10% loss."""
  loss = 0.2 * account_size
  return account_size - loss


#!/usr/bin/env python
# -*- coding: UTF8 -*-
"""https://oxfordstrat.com/coasdfASD32/uploads/2016/01/turtle-rules.pdf

Position Sizing
The Turtles used a volatility-based constant percentage risk position 
sizing algorithm.

Position sizing is one of the most important but least understood components
of any trading system.

The Turtles used a position sizing algorithm that was very advanced for its day,
because it normalized the dollar volatility of a position by adjusting the position size
based on the dollar volatility of the market. This meant that a given position would
tend to move up or down in a given day about the same amount in dollar terms (when
compared to positions in other markets), irrespective of the underlying volatility of the
particular market.

This is true because positions in markets that moved up and down a large amount per
contract would have an offsetting smaller number of contracts than positions in
markets that had lower volatility.

This volatility normalization is very important because it means that different trades in
different markets tend to have the same chance for a particular dollar loss or a
particular dollar gain. This increased the effectiveness of the diversification of trading
across many markets.

Even if the volatility of a given market was lower, any significant trend would result in
a sizeable win because the Turtles would have held more contracts of that lower
volatility commodity.
"""
import math
import pandas as pd

from yahoo_fin.stock_info import get_data, get_live_price


class Unit(object):
  """A Volatitility Adjusted Position Unit. 
  Args:
    asset: The ticker or symbol of an asset.
    account_size: The total value of the account.
  """
  def __init__(self, asset: str, account_size: float) -> None:
    self.asset = asset
    self.account_size = account_size
    self.__process()
    
  def __process(self):
    self.calculate_N()
    self.get_unit()

  def __str__(self):
    return f"{self.asset}: {self.unit}"


  """Volatility - The Meaning of N
  The Turtles used a concept that Richard Dennis and Bill Eckhardt called N to
  represent the underlying volatility of a particular market.

  N is simply the 20-day expontential moving average of the True Range, which is now
  more commonly known as the ATR. Conceptually, N represents the average range in
  price movement that a particular market makes in a single day, accounting for opening
  gaps. N was measured in the same points as the underlying contract.
  """

  """
  To compute the daily true range:
    TRUE RANGE = Maximum(H-L, H-PDC, PDC-L)
  """
  def _true_range(self) -> pd.DataFrame:
    """Add true range column to dataframe."""
    df = get_data(ticker=self.asset, interval='1d')

    dataframe = df[['low', 'high', 'close']].tail(21)
    dataframe['previous_close'] = dataframe['close'].shift(1)
    dataframe['true_range'] = 0

    for index, row in enumerate(dataframe.iterrows()):
      curr = dataframe.iloc[index]
      dataframe.iloc[index, dataframe.columns.get_loc('true_range')] = max(curr['high'] - curr['low'], curr['high'] - curr['previous_close'], curr['previous_close'] - curr['low'])

    self.dataframe = dataframe
    return dataframe


  """
  Since this formula requires a previous day's N value, you must start with a 20-day
  simple average of the True Rnage for the initial calculation.
  """
  def _pdn(self) -> float:
    """Calculate the Previous Day's N."""
    return (sum(self.dataframe['true_range'][1:])) / 20
  

  """
  To compute N use the following formula:
    N = (19 * PDN + TR) / 20
  """
  def calculate_N(self) -> float:
    """Calculate N."""
    dataframe = self._true_range()
    pdn = self._pdn()
    true_range = dataframe['true_range'][-1]

    self.N = (19 * pdn + true_range) / 20
    return self.N

  
  """Dollar Volatility Adustment
  The first step in determining the positino size was to determine the dollar volatility
  represented by the underlying market's price volatility (defined by its N).
  """

  """
  This sounds more complicated than it is. It is determined using the simple formula:
    Dollar Volatility = N * Dollars per Point
  """
  def _dollar_volatility(self) -> float:
    """Calculate the market dollar volatility of the asset."""
    # add in functionality for futures contract.
    pps = get_live_price(self.asset)
    return self.N * pps


  """Volatility Adjusted Position Units
  The Turtles build positions in pieces which we called Units. Units were sized so that 1
  N represented 1% of the account equity.
  """
  
  """
  Thus, a unit for a given market or commodity can be calculated using the following
  formula:
    Unit = 1% of Account / Market Dollar Volatility
    or
    Unit = 1% of Account / (N * Dollars per Point)
  """
  def get_unit(self) -> float:
    """Calculate the volatility adjusted unit size."""
    dollar_volatility = self._dollar_volatility()
    self.unit = math.trunc((0.01 * self.account_size) / dollar_volatility)
    return self.unit


#!/usr/bin/env python
# -*- coding: UTF8 -*-
"""https://oxfordstrat.com/coasdfASD32/uploads/2016/01/turtle-rules.pdf

Units as a measure of Risk

Since the Turtles used the Unit as the base measure for position size, and since those
units were volatility risk adjusted, the Unit was a measure of both the risk of a position,
and of the entire portfolio of positions.

The Turtles were given risk management rules that limited the number of Units that
we could maintain at any given time, on four different levels. In essence, these rules
controlled the risk that a trader could carry, and these limits minimized losses
during prolonged losing periods, as well as during extraordinary price movements.

An example of an extraordinary price movement was the day after the October, 1987
stock market crash. The U.S. Federal Reserve lowered interest rates by several
percentage points overnight to boost the confidence of the stock market and the
country. The Turtles were loaded long in interest rate futures: Eurodollars, TBills and
Bonds. The losses the following day were enormous. In some cases, 20% to 40% of
account equity was lost in a single day. But these losses would have been
correspondingly higher without the maximum position limits.

The limits were: 
Level    Type                                Maximum Units
1        Single Market                       4 units
2        Closely Correlated Markets          6 units
3        Loosely Correlated Markets          10 units
4        Single Direction -- Long or Short   12 units
"""


"""Single Markets - A maximum of four Units per market."""
class SingleMarketsException(Exception):
    pass

"""Closely Correlated Markets -
For markets that were closely correlated and there could be
a maximum of 6 Units in one particular direction (i.e. 6 long units or 6 short units).
Closely correlated markets include: heating oil and crude oil; gold and silver; Swiss
franc and Deutschmark; TBill and Eurodollar, etc.
"""
class CloselyCorrelatedMarketsException(Exception):
    pass

"""Loosely Correlated Markets - 
For loosely correlated markets, there could be a
maximum of 10 Units in one particular direction. Loosely correlated markets included:
gold and copper; silver and copper, and many grain combinations that the Turtles did
not trade because of positions limits.
"""
class LooselyCorrelatedMarketsException(Exception):
    pass

"""Single Direction - 
The maximum number of total Units in one direction long or short
was 12 Units. Thus, one could theoretically have had 12 Units long and 12 Units short
at the same time.
"""
class SingleDirectionException(Exception):
    pass